#include "menuPar.h"

menuPar::menuPar(QWidget *parent)
	: QMainWindow(parent)
{
	setupUi(this); 
 
	parser_Gui = new GUIPar(this);
	readSettings();
	setCentralWidget(parser_Gui);
	connect(parser_Gui, SIGNAL(modified(bool)),	this, SLOT(setWindowChanged(bool)));

	connect(actionClear_log, SIGNAL(triggered()), this, SLOT (clear_log()));
	connect(actionClear_edit, SIGNAL(triggered()), this, SLOT (clear_edit()));
	connect(actionClear_history, SIGNAL(triggered()), this, SLOT (clear_history()));
	connect(actionExit, SIGNAL(triggered()), qApp, SLOT(closeAllWindows()));
	connect(actionAbout, SIGNAL(triggered()), this, SLOT(about()));
	connect(menuPar::action_Copy_result, SIGNAL(triggered()), this, SLOT(copy()));

	createToolBar();

	saveFileFilter = "*.doc";
	
	setCurrentFile(tr("Untitled.doc"));

	statusSet = new QLabel();
	statusSet->setFrameStyle(QFrame::Box | QFrame::Raised);
	
	statusBar()->addPermanentWidget(statusSet);
	updateStatusBar();

	saveOk = false;
}
void menuPar::setWindowChanged(bool ok)
{
	setWindowModified(ok);
	updateStatusBar();
}
void menuPar::createToolBar()
{
	connect(actionSave_Log_as, SIGNAL(triggered()), this, SLOT(saveLog_as()));
	connect(actionSave_Log, SIGNAL(triggered()), this, SLOT(saveLog()));
	connect(actionSettings, SIGNAL(triggered()), this, SLOT(settings()));
}
bool menuPar::saveLog()
{
	if(saveOk == false)
	{
		return saveLog_as();
	}
	else if(!parser_Gui->saveLog(curFile))	
	{
		QMessageBox::critical(this, tr("Parser"), tr("Can`t save file"));
		statusBar()->showMessage(tr("Can`t save file"), 2000);
		return false;
	}
	setCurrentFile(curFile);
	return true;	
}
bool menuPar::saveLog_as()
{	   

	QString fname = QFileDialog::getSaveFileName(this, tr("Save"), curFile, saveFileFilter);
	if(!fname.isNull())
	{
		if(!parser_Gui->saveLog(fname)) 
		{
			QMessageBox::critical(this,tr("Parser"), tr("Can`t save file"));
			statusBar()->showMessage(tr("Can`t save file"), 2000);
			return false;
		}
		else 
		{
			setCurrentFile(fname);

			saveOk = true;

			statusBar()->showMessage(tr("File saved"), 2000);
			return true;
		}
	}
	else 
	{
		statusBar()->showMessage(tr("Saving canceled"), 2000);
        return false;
    }
}
void menuPar::settings()
{
	set = new Settings_dialog(parser_Gui->par->options,this);
	
	connect(set -> buttonBox, SIGNAL(clicked(QAbstractButton *)), this, SLOT(setDefault_settings_dialog(QAbstractButton *)));

	int code = set->exec();
	
	switch(code)
	{
		case QDialog::Accepted:

			if(parser_Gui->par->options != set->getSet())
			{
				parser_Gui->par->options = set->getSet();
				parser_Gui->setChecked_for_checkBoxs();

				statusBar()->showMessage(tr("Settings accepted"), 2000);
				updateStatusBar();
			}	
			break;
		case QDialog::Rejected:
			statusBar()->showMessage(tr("Settings rejected"), 2000);
			break;
	}
	delete set;
}
void menuPar::clear_log()
{
	parser_Gui->clearQTextEdit();
	parser_Gui->removeFocus();
	status->showMessage(tr("Log cleared"), 2000);
}
void menuPar::clear_edit()
{
	parser_Gui->clearQLineEdit();
	parser_Gui->removeFocus();
	statusBar()->showMessage(tr("Lineedit cleared"), 2000);
}
void menuPar::clear_history()
{
	parser_Gui->clearQListWidget();
	parser_Gui->removeFocus();
	statusBar()->showMessage(tr("History cleared"), 2000);
}
void menuPar::about()
{
	QMessageBox::about(this, tr("About Parser"),
			tr("<h2>Parser 1.0</h2>"
				"<p>Copyright &copy; 2009."
				"<p>Parser is a classic calculator with more extencive means:<BR> - calculation with formula;<BR> - support calculation with variables;<BR> - save the data (log) to disk."));
}
bool menuPar::okToContinue()
{
	if (isWindowModified ()) 
	{
		int ok = QMessageBox::warning(this, tr("Parser"), 
			QString(tr("Do you want to save your changes in ") + QFileInfo(curFile).fileName() + " ?"), QMessageBox::Save | QMessageBox::No | QMessageBox::Cancel);
		if (ok == QMessageBox::Save) 
		{
			return saveLog();
		} 
		else if (ok == QMessageBox::Cancel) return false;
	}
	return true;
}
void menuPar::closeEvent(QCloseEvent *event)
{
	if (okToContinue()) 
	{
		writeSettings();
		event->accept();
	} 
	else event->ignore();
}
void menuPar::setCurrentFile(const QString &fileName)
{
	curFile = fileName;
	setWindowModified(false);

	setWindowTitle(tr("%1[*] - %2").arg(QFileInfo(curFile).completeBaseName ()).arg(tr("Parser")));
}
void menuPar::readSettings()
{
   QSettings settings("Parser", "Settings");

   restoreGeometry(settings.value("geometry").toByteArray());
   parser_Gui->par->options.operationInLog = settings.value("operations in log", false).toBool();
   parser_Gui->par->options.formulaInLog = settings.value("formula in log", true).toBool();;
   parser_Gui->par->options.resultInLog = settings.value("result in log", true).toBool();;
   parser_Gui->par->options.on_rad = settings.value("on_rad", false).toBool();;
   parser_Gui->par->options.precision = settings.value("precision", 5).toUInt();;
   parser_Gui->par->options.type_number = settings.value("type_number", 'g').toChar().toAscii();;

   parser_Gui->setChecked_for_checkBoxs();
}
void menuPar::writeSettings()
{
    QSettings settings("Parser", "Settings");

    settings.setValue("geometry", saveGeometry());
	settings.setValue("operations in log", parser_Gui->par->options.operationInLog);
	settings.setValue("formula in log", parser_Gui->par->options.formulaInLog);
	settings.setValue("result in log", parser_Gui->par->options.resultInLog);
	settings.setValue("on_rad", parser_Gui->par->options.on_rad);
	settings.setValue("precision", parser_Gui->par->options.precision);
	settings.setValue("type_number", parser_Gui->par->options.type_number);
}
void menuPar::setDefault_settings_dialog( QAbstractButton * button )
{
	if ((QPushButton*)button == set->buttonBox->button(QDialogButtonBox::RestoreDefaults))
	{	
		set->set.DefaultSettings();
		set->setWidget_status();
	}
}
void menuPar::updateStatusBar()
{
	statusSet->setText("ans="+QString::number(parser_Gui->par->calculation(), parser_Gui->par->options.type_number, parser_Gui->par->options.precision)+QString(tr(" | type: ") + QString(parser_Gui->par->options.type_number) + tr(" | precision=") + QString::number(parser_Gui->par->options.precision) + " "));
}
void menuPar::copy()
{
	if(!parser_Gui->resultEdit->text().isEmpty())
	{
		parser_Gui->resultEdit->selectAll();	
		parser_Gui->resultEdit->copy();
		statusBar()->showMessage(tr("Current result copied"), 2000);
		parser_Gui->resultEdit->deselect();
	}
	else statusBar()->showMessage(tr("Can`t copy current result"), 2000);
}